/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail;

import javax.servlet.jsp.tagext.TagData;
import javax.servlet.jsp.tagext.TagExtraInfo;
import javax.servlet.jsp.tagext.VariableInfo;


/**
 * <p>
 * Defines a tag extra info which describes the attribute to variable mapping that will occur within the
 * <code>BreadCrumbTrailTag</code>. And this tag extra info class should be specified in the corresponding .tld file.
 * </p>
 *
 * <p>
 * This tag extra info will define two variables (that can be used in the body of the <code>BreadCrumbTrailTag</code>):
 *
 * <ol>
 * <li>
 * <code>BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE</code> variable that will contain a reference to the
 * <code>BreadCrumbNode</code> that should be formatted.
 * </li>
 * <li>
 * <code>BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS</code> variable that will contain a reference to the string
 * that represents the style that will be used. An empty string or <code>null</code> string will represent no styles
 * to be applied.
 * </li>
 * </ol>
 * </p>
 *
 * <p>
 * <b>Thread Safety: </b> this class is immutable and therefore thread safe.
 * </p>
 *
 * @author Pops, TCSDEVELOPER
 * @version 2.0
 */
public class BreadCrumbTrailTagExtraInfo extends TagExtraInfo {
    /**
     * <p>
     * Defines the variables that can be used in the body of the <code>BreadCrumbTrailTag</code>.
     * </p>
     *
     * @param data A non-null tag data to use. It will not be used in this implementation.
     *
     * @return Always a 2 element array representing the two elements to define.
     */
    public VariableInfo[] getVariableInfo(TagData data) {
        return new VariableInfo[] {
            new VariableInfo(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, BreadCrumbNode.class.getName(), true,
                VariableInfo.NESTED),
            new VariableInfo(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS, String.class.getName(), true,
                VariableInfo.NESTED)
        };
    }
}
