/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.nodeformatter;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbException;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbTrailTag;
import com.topcoder.web.ui.tag.breadcrumbtrail.Helper;

import java.io.IOException;

import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.TagSupport;


/**
 * <p>
 * This tag will format the node and title using the html anchor pattern of &quot;&lt;a
 * href='node.getURL()'&gt;title&lt;/a&gt;
 * </p>
 *
 * <p>
 * <strong>Thread Safety:</strong> This class is thread safe in the context of the JSP rendering process.
 * </p>
 *
 * @author Pops, TCSDEVELOPER
 * @version 2.0
 */
public class HtmlAnchorNodeFormatterTag extends TagSupport {
    /**
     * <p>
     * Creates a new instance of <code>HtmlAnchorNodeFormatterTag</code> class.
     * </p>
     */
    public HtmlAnchorNodeFormatterTag() {
        super();
    }

    /**
     * <p>
     * This method will format the current node and style as a simple html anchor and then write the results out.
     * </p>
     *
     * @return Always SKIP_BODY.
     *
     * @throws BreadCrumbException if any exception occurs.
     */
    public int doStartTag() throws BreadCrumbException {
        // get the node to format
        Object node = pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, PageContext.PAGE_SCOPE);

        if (!(node instanceof BreadCrumbNode)) {
            throw new BreadCrumbException("The node to format can not be found.");
        }

        // get the style to format
        Object style = pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS,
                PageContext.PAGE_SCOPE);

        if ((style != null) && !(style instanceof String)) {
            throw new BreadCrumbException("The style to format is not type of String.");
        }

        style = Helper.convertEmptyString((String) style);

        // create the string buffer
        StringBuffer buffer = new StringBuffer();
        buffer.append("<a");

        if (style != null) {
            buffer.append(" class=\"" + style + "\"");
        }

        buffer.append(" href=\"").append(((BreadCrumbNode) node).getUrl()).append("\">");
        buffer.append((((BreadCrumbNode) node).getTitle() == null)
                ? "Unspecified" : ((BreadCrumbNode) node).getTitle());
        buffer.append("</a>");

        JspWriter writer = null;

        try {
            writer = pageContext.getOut();
            writer.print(buffer.toString());
            writer.flush();
        } catch (IOException e) {
            throw new BreadCrumbException("Errors occur when writing the content.", e);
        } finally {
            if (writer != null) {
                try {
                    writer.close();
                } catch (IOException e) {
                    // ignore
                }
            }
        }

        return SKIP_BODY;
    }
}
