/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail;

import junit.framework.TestCase;

import javax.servlet.jsp.JspTagException;


/**
 * <p>
 * Unit test cases for <code>BreadCrumbException</code>.
 * </p>
 *
 * <p>
 * This class is pretty simple. The test cases simply verifies the exception can be instantiated with the error message
 * and cause properly propagated, and that it comes with correct inheritance.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 2.0
 */
public class BreadCrumbExceptionUnitTest extends TestCase {
    /**
     * <p>
     * The error message used for testing.
     * </p>
     */
    private static final String ERROR_MESSAGE = "test error message";

    /**
     * <p>
     * Creation test.
     * </p>
     *
     * <p>
     * Verifies the error message is properly propagated.
     * </p>
     */
    public void testBreadCrumbException1() {
        BreadCrumbException ce = new BreadCrumbException(ERROR_MESSAGE);

        assertNotNull("Unable to instantiate BreadCrumbException.", ce);
        assertEquals("Error message is not properly propagated to super class.", ERROR_MESSAGE, ce.getMessage());
    }

    /**
     * <p>
     * Creation test.
     * </p>
     *
     * <p>
     * Verifies the error message and the cause are properly propagated.
     * </p>
     */
    public void testBreadCrumbException2() {
        Throwable cause = new Exception();
        BreadCrumbException ce = new BreadCrumbException(ERROR_MESSAGE, cause);

        assertNotNull("Unable to instantiate BreadCrumbException.", ce);
        assertEquals("Cause is not properly propagated to super class.", cause, ce.getRootCause());
    }

    /**
     * <p>
     * Inheritance test.
     * </p>
     *
     * <p>
     * Verifies BreadCrumbException subclasses JspTagException.
     * </p>
     */
    public void testBreadCrumbExceptionInheritance() {
        assertTrue("BreadCrumbException does not subclass JspTagException.",
            new BreadCrumbException(ERROR_MESSAGE) instanceof JspTagException);
    }

    /**
     * <p>
     * Inheritance test.
     * </p>
     *
     * <p>
     * Verifies BreadCrumbException subclasses JspTagException.
     * </p>
     */
    public void testBreadCrumbExceptionInheritance2() {
        assertTrue("BreadCrumbException does not subclass JspTagException.",
            new BreadCrumbException(ERROR_MESSAGE, new Exception()) instanceof JspTagException);
    }
}
