/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.nodediscovery;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbTrailTag;

import org.apache.cactus.JspTestCase;

import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.BodyContent;
import javax.servlet.jsp.tagext.BodyTagSupport;


/**
 * <p>
 * Tests functionality and error cases of <code>InlineTextNodeDiscoveryTag</code> class.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 2.0
 */
public class InlineTextNodeDiscoveryTagUnitTest extends JspTestCase {
    /** Represents the <code>InlineTextNodeDiscoveryTag</code> instance used for testing. */
    private InlineTextNodeDiscoveryTag nodeDiscoveryTag = null;

    /**
     * <p>
     * Sets up the test environment. The test instance is created.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    protected void setUp() throws Exception {
        nodeDiscoveryTag = new InlineTextNodeDiscoveryTag();
    }

    /**
     * <p>
     * Tests the accuracy of constructor <code>InlineTextNodeDiscoveryTag()</code>.
     * </p>
     */
    public void testInlineTextNodeDiscoveryTag_Accuracy() {
        assertNotNull("The InlineTextNodeDiscoveryTag instance should be created.", nodeDiscoveryTag);
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doStartTag()</code>.
     * </p>
     *
     * @throws Exception any exception to JUnit.
     */
    public void testDoStartTag_Accuracy() throws Exception {
        nodeDiscoveryTag.setPageContext(this.pageContext);
        assertEquals("DoStartTag failed", BodyTagSupport.EVAL_BODY_BUFFERED, nodeDiscoveryTag.doStartTag());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doAfterBody()</code> with null bodyContent, JspTagException is expected.
     * </p>
     *
     * @throws Exception any exception to JUnit.
     */
    public void testDoAfterBody_NullBodyContent() throws Exception {
        nodeDiscoveryTag.setPageContext(this.pageContext);

        try {
            nodeDiscoveryTag.doAfterBody();
            fail("JspTagException should be thrown.");
        } catch (JspTagException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doAfterBody()</code> with empty bodyContent, JspTagException is expected.
     * </p>
     *
     * @throws Exception any exception to JUnit.
     */
    public void testDoAfterBody_EmptyBodyContent() throws Exception {
        BodyContent bodyContent = pageContext.pushBody();
        bodyContent.print("  ");
        nodeDiscoveryTag.setPageContext(this.pageContext);
        nodeDiscoveryTag.setBodyContent(bodyContent);

        try {
            nodeDiscoveryTag.doAfterBody();
            fail("JspTagException should be thrown.");
        } catch (JspTagException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doAfterBody()</code>.
     * </p>
     *
     * @throws Exception any exception to JUnit.
     */
    public void testDoAfterBody_Accuracy() throws Exception {
        BodyContent bodyContent = pageContext.pushBody();
        String url = "url";
        bodyContent.print(url);
        nodeDiscoveryTag.setPageContext(this.pageContext);
        nodeDiscoveryTag.setBodyContent(bodyContent);
        assertEquals("DoStartTag failed", BodyTagSupport.SKIP_BODY, nodeDiscoveryTag.doAfterBody());

        // The current node should be saved as an attribute into the current page context using page scope.
        Object object = this.pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE,
                PageContext.PAGE_SCOPE);
        assertTrue("The current node should be saved as an attribute into the current page context using page scope.",
            object instanceof BreadCrumbNode);
        assertEquals("The current node should be saved properly.", null, ((BreadCrumbNode) object).getTitle());
        assertEquals("The current node should be saved properly.", url, ((BreadCrumbNode) object).getUrl());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getTitle()</code>.
     * </p>
     */
    public void testGetTitle_Accuracy() {
        assertEquals("The title should be got properly.", null, nodeDiscoveryTag.getTitle());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>setTitle(String)</code> with <code>null</code> title.
     * </p>
     */
    public void testSetTitle_NullTitle() {
        nodeDiscoveryTag.setTitle(null);
        assertEquals("The title should be set properly.", null, nodeDiscoveryTag.getTitle());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>setTitle(String)</code> with empty title.
     * </p>
     */
    public void testSetTitle_EmptyTitle() {
        nodeDiscoveryTag.setTitle(" ");
        assertEquals("The title should be set properly.", null, nodeDiscoveryTag.getTitle());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>setTitle(String)</code>.
     * </p>
     */
    public void testSetTitle_Accuracy() {
        String title = "title";
        nodeDiscoveryTag.setTitle(title);
        assertEquals("The title should be set properly.", title, nodeDiscoveryTag.getTitle());
    }
}
