/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.nodeformatter;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbException;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbTrailTag;
import com.topcoder.web.ui.tag.breadcrumbtrail.UnitTestHelper;

import org.apache.cactus.JspTestCase;

import java.util.HashMap;

import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.Tag;


/**
 * <p>
 * Tests functionality and error cases of <code>TemplateNodeFormatterTag</code> class.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 2.0
 */
public class TemplateNodeFormatterTagUnitTest extends JspTestCase {
    /** Represents the template used to create the <code>TemplateNodeFormatterTag</code> instance for testing. */
    private static final String TEMPLATE = "template";

    /** Represents the <code>TemplateNodeFormatterTag</code> instance used for testing. */
    private TemplateNodeFormatterTag formatter = null;

    /**
     * <p>
     * Sets up the test environment. The test instance is created.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    protected void setUp() throws Exception {
        formatter = new TemplateNodeFormatterTag(TEMPLATE);
    }

    /**
     * <p>
     * Tests the accuracy of constructor <code>TemplateNodeFormatterTag()</code>.
     * </p>
     */
    public void testTemplateNodeFormatterTag_Accuracy() {
        formatter = new TemplateNodeFormatterTag();
        assertNotNull("The TemplateNodeFormatterTag instance should be created.", formatter);
    }

    /**
     * <p>
     * Tests the accuracy of constructor <code>TemplateNodeFormatterTag(String)</code>.
     * </p>
     */
    public void testTemplateNodeFormatterTag_StringAccuracy() {
        assertNotNull("The TemplateNodeFormatterTag instance should be created.", formatter);
        assertEquals("The template should be set properly.", TEMPLATE,
            UnitTestHelper.getPrivateField(formatter.getClass(), formatter, "template"));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getTemplate()</code>.
     * </p>
     */
    public void testGetTemplate_Accuracy() {
        assertEquals("The template should be got properly.", TEMPLATE, formatter.getTemplate());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>setTemplate(String)</code>.
     * </p>
     */
    public void testSetTemplate_Accuracy() {
        formatter.setTemplate(null);
        assertEquals("The template should be set properly.", null,
            UnitTestHelper.getPrivateField(formatter.getClass(), formatter, "template"));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doStartTag()</code> when the template is missing, SKIP_BODY should be
     * returned.
     * </p>
     *
     * @throws Exception any exception to JUnit.
     */
    public void testDoStartTag_MissingTemplate() throws Exception {
        formatter.setTemplate(null);
        assertEquals("DoStartTag failed", Tag.SKIP_BODY, formatter.doStartTag());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doStartTag()</code> when the template is empty, SKIP_BODY should be returned.
     * </p>
     *
     * @throws Exception any exception to JUnit.
     */
    public void testDoStartTag_EmptyTemplate() throws Exception {
        formatter.setTemplate(" ");
        assertEquals("DoStartTag failed", Tag.SKIP_BODY, formatter.doStartTag());
    }

    /**
     * <p>
     * Tests the method <code>doStartTag()</code> when the BreadCrumbNode is missing, expect BreadCrumbException.
     * </p>
     *
     * @throws Exception any exception to JUnit.
     */
    public void testDoStartTag_MissingBreadCrumbNode() throws Exception {
        this.pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS, "style",
                PageContext.PAGE_SCOPE);
        formatter.setPageContext(this.pageContext);

        try {
            formatter.doStartTag();
            fail("BreadCrumbException should be thrown.");
        } catch (BreadCrumbException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the method <code>doStartTag()</code> when the BreadCrumbNode is not valid type, expect
     * BreadCrumbException.
     * </p>
     *
     * @throws Exception any exception to JUnit.
     */
    public void testDoStartTag_NotValidBreadCrumbNode() throws Exception {
        this.pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, "node", PageContext.PAGE_SCOPE);
        this.pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS, "style",
                PageContext.PAGE_SCOPE);
        formatter.setPageContext(this.pageContext);

        try {
            formatter.doStartTag();
            fail("BreadCrumbException should be thrown.");
        } catch (BreadCrumbException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the method <code>doStartTag()</code> when the style attribute is not valid type, expect
     * BreadCrumbException.
     * </p>
     *
     * @throws Exception any exception to JUnit.
     */
    public void testDoStartTag_NotValidStyle() throws Exception {
        this.pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, new BreadCrumbNode("url", "title"),
            PageContext.PAGE_SCOPE);
        this.pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS, new HashMap(),
            PageContext.PAGE_SCOPE);
        formatter.setPageContext(this.pageContext);

        try {
            formatter.doStartTag();
            fail("BreadCrumbException should be thrown.");
        } catch (BreadCrumbException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doStartTag()</code> when the style attribute is an empty string.
     * </p>
     *
     * @throws Exception any exception to JUnit.
     */
    public void testDoStartTag_EmptyStyleAccuracy() throws Exception {
        BreadCrumbNode node = new BreadCrumbNode("url", "title");
        this.pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, node, PageContext.PAGE_SCOPE);
        this.pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS, "  ", PageContext.PAGE_SCOPE);
        formatter.setPageContext(this.pageContext);
        assertEquals("DoStartTag failed", Tag.SKIP_BODY, formatter.doStartTag());
        assertEquals("The node should be inserted in to the request's attributes list.", node,
            pageContext.getRequest().getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE));
        assertEquals("The style should be inserted in to the request's attributes list.", null,
            pageContext.getRequest().getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doStartTag()</code> when the style attribute is missing.
     * </p>
     *
     * @throws Exception any exception to JUnit.
     */
    public void testDoStartTag_MissingStyleAccuracy() throws Exception {
        BreadCrumbNode node = new BreadCrumbNode("url", "title");
        this.pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, node, PageContext.PAGE_SCOPE);
        formatter.setPageContext(this.pageContext);
        assertEquals("DoStartTag failed", Tag.SKIP_BODY, formatter.doStartTag());
        assertEquals("The node should be inserted in to the request's attributes list.", node,
            pageContext.getRequest().getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE));
        assertEquals("The style should be inserted in to the request's attributes list.", null,
            pageContext.getRequest().getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doStartTag()</code>.
     * </p>
     *
     * @throws Exception any exception to JUnit.
     */
    public void testDoStartTag_Accuracy() throws Exception {
        BreadCrumbNode node = new BreadCrumbNode("url", "title");
        String style = "style";
        this.pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, node, PageContext.PAGE_SCOPE);
        this.pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS, style, PageContext.PAGE_SCOPE);
        formatter.setPageContext(this.pageContext);
        assertEquals("DoStartTag failed", Tag.SKIP_BODY, formatter.doStartTag());
        assertEquals("The node should be inserted in to the request's attributes list.", node,
            pageContext.getRequest().getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE));
        assertEquals("The style should be inserted in to the request's attributes list.", style,
            pageContext.getRequest().getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS));
    }
}
