/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;
import com.topcoder.web.ui.tag.breadcrumbtrail.UnitTestHelper;
import com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery.nodematchers.RegexNodeMatcher;

import junit.framework.TestCase;


/**
 * <p>
 * Tests functionality and error cases of <code>NodePair</code> class.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 2.0
 */
public class NodePairUnitTest extends TestCase {
    /** Represents the <code>RegexNodeMatcher</code> instance used for testing. */
    private RegexNodeMatcher nodeMatcher = null;

    /** Represents the <code>BreadCrumbNode</code> instance used for testing. */
    private BreadCrumbNode node = null;

    /** Represents the <code>NodePair</code> instance used for testing. */
    private NodePair nodePair = null;

    /**
     * <p>
     * Sets up the test environment. The test instance is created.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    protected void setUp() throws Exception {
        node = new BreadCrumbNode("url", "title");
        nodeMatcher = new RegexNodeMatcher("title", "url", ".*");
        nodePair = new NodePair(node, nodeMatcher);
    }

    /**
     * <p>
     * Tests the constructor <code>NodePair(BreadCrumbNode, NodeMatcher)</code> with null node,
     * IllegalArgumentException should be thrown.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testNodePair_NullNode() throws Exception {
        try {
            new NodePair(null, nodeMatcher);
            fail("IllegalArgumentException should be thrown");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the constructor <code>NodePair(BreadCrumbNode, NodeMatcher)</code> with null nodeMatcher, no exception
     * should be thrown.
     * </p>
     */
    public void testNodePair_NullNodeMatcher() {
        nodePair = new NodePair(node, null);

        assertNotNull("The NodePair instance should be created.", nodePair);
        assertEquals("The node should be set properly.", node,
            UnitTestHelper.getPrivateField(nodePair.getClass(), nodePair, "node"));
        assertEquals("The nodeMatcher should be set properly.", null,
            UnitTestHelper.getPrivateField(nodePair.getClass(), nodePair, "nodeMatcher"));
    }

    /**
     * <p>
     * Tests the accuracy of constructor <code>NodePair(BreadCrumbNode, NodeMatcher)</code>.
     * </p>
     */
    public void testNodePair_Accuracy() {
        nodePair = new NodePair(node, nodeMatcher);

        assertNotNull("The NodePair instance should be created.", nodePair);
        assertEquals("The node should be set properly.", node,
            UnitTestHelper.getPrivateField(nodePair.getClass(), nodePair, "node"));
        assertEquals("The nodeMatcher should be set properly.", nodeMatcher,
            UnitTestHelper.getPrivateField(nodePair.getClass(), nodePair, "nodeMatcher"));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getNode()</code> when the node's title is null.
     * </p>
     */
    public void testGetNode_NullNodeTitleAccuracy() {
        node = new BreadCrumbNode("nodeUrl", null);
        nodePair = new NodePair(node, nodeMatcher);
        assertEquals("The node should be got properly.", node.getUrl(), nodePair.getNode().getUrl());
        assertEquals("The node should be got properly.", nodeMatcher.getTitle(), nodePair.getNode().getTitle());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getNode()</code> when the node's title is null and nodeMatcher is null.
     * </p>
     */
    public void testGetNode_NullNodeTitleAndNullNodeMatcherAccuracy() {
        node = new BreadCrumbNode("nodeUrl", null);
        nodePair = new NodePair(node, null);
        assertEquals("The node should be got properly.", node, nodePair.getNode());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getNode()</code>.
     * </p>
     */
    public void testGetNode_Accuracy() {
        assertEquals("The node should be got properly.", node, nodePair.getNode());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getNodeMatcher()</code>.
     * </p>
     */
    public void testGetNodeMatcher_Accuracy() {
        assertEquals("The nodeMatcher should be got properly.", nodeMatcher, nodePair.getNodeMatcher());
    }
}
