/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery.nodematchers;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbException;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;
import com.topcoder.web.ui.tag.breadcrumbtrail.UnitTestHelper;

import junit.framework.TestCase;

import java.util.regex.Pattern;


/**
 * <p>
 * Tests functionality and error cases of <code>RegexNodeMatcher</code> class.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 2.0
 */
public class RegexNodeMatcherUnitTest extends TestCase {
    /** Represents the title used to create the <code>RegexNodeMatcher</code> instance for testing. */
    private static final String TITLE = "title";

    /** Represents the url used to create the <code>RegexNodeMatcher</code> instance for testing. */
    private static final String URL = "url";

    /** Represents the nodePattern used to create the <code>RegexNodeMatcher</code> instance for testing. */
    private static final String NODE_PATTERN = "a.*";

    /** Represents the <code>RegexNodeMatcher</code> instance used for testing. */
    private RegexNodeMatcher nodeMatcher = null;

    /**
     * <p>
     * Sets up the test environment. The test instance is created.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    protected void setUp() throws Exception {
        nodeMatcher = new RegexNodeMatcher(TITLE, URL, NODE_PATTERN);
    }

    /**
     * <p>
     * Tests the constructor <code>RegexNodeMatcher(String, String, String)</code> with null title,
     * IllegalArgumentException should be thrown.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testRegexNodeMatcher_NullTitle() throws Exception {
        try {
            new RegexNodeMatcher(null, URL, NODE_PATTERN);
            fail("IllegalArgumentException should be thrown");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the constructor <code>RegexNodeMatcher(String, String, String)</code> with empty title,
     * IllegalArgumentException should be thrown.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testRegexNodeMatcher_EmptyTitle() throws Exception {
        try {
            new RegexNodeMatcher(" ", URL, NODE_PATTERN);
            fail("IllegalArgumentException should be thrown");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the constructor <code>RegexNodeMatcher(String, String, String)</code> with null url,
     * IllegalArgumentException should be thrown.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testRegexNodeMatcher_NullUrl() throws Exception {
        try {
            new RegexNodeMatcher(TITLE, null, NODE_PATTERN);
            fail("IllegalArgumentException should be thrown");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the constructor <code>RegexNodeMatcher(String, String, String)</code> with empty url,
     * IllegalArgumentException should be thrown.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testRegexNodeMatcher_EmptyUrl() throws Exception {
        try {
            new RegexNodeMatcher(TITLE, " ", NODE_PATTERN);
            fail("IllegalArgumentException should be thrown");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the constructor <code>RegexNodeMatcher(String, String, String)</code> with null nodePattern,
     * IllegalArgumentException should be thrown.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testRegexNodeMatcher_NullNodePattern() throws Exception {
        try {
            new RegexNodeMatcher(TITLE, URL, null);
            fail("IllegalArgumentException should be thrown");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the constructor <code>RegexNodeMatcher(String, String, String)</code> with empty nodePattern,
     * IllegalArgumentException should be thrown.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testRegexNodeMatcher_EmptyNodePattern() throws Exception {
        try {
            new RegexNodeMatcher(TITLE, URL, " ");
            fail("IllegalArgumentException should be thrown");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the constructor <code>RegexNodeMatcher(String, String, String)</code> with invalid nodePattern,
     * BreadCrumbException should be thrown.
     * </p>
     */
    public void testRegexNodeMatcher_InvalidNodePattern() {
        try {
            new RegexNodeMatcher(TITLE, URL, "**");
            fail("BreadCrumbException should be thrown");
        } catch (BreadCrumbException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the accuracy of constructor: <code>RegexNodeMatcher(String, String, String)</code>.
     * </p>
     */
    public void testAbstractNodeMatcher_Accuracy() {
        assertNotNull("The RegexNodeMatcher instance should be created.", nodeMatcher);
        assertEquals("The url should be set properly.", URL,
            UnitTestHelper.getPrivateField(nodeMatcher.getClass().getSuperclass(), nodeMatcher, "url"));
        assertEquals("The title should be set properly.", TITLE,
            UnitTestHelper.getPrivateField(nodeMatcher.getClass().getSuperclass(), nodeMatcher, "title"));
        assertEquals("The nodePattern should be set properly.", NODE_PATTERN,
            UnitTestHelper.getPrivateField(nodeMatcher.getClass(), nodeMatcher, "nodePattern"));
        assertEquals("The pattern should be set properly.", NODE_PATTERN,
            ((Pattern) UnitTestHelper.getPrivateField(nodeMatcher.getClass(), nodeMatcher, "pattern")).pattern());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>matches(BreadCrumbNode)</code> with null node.
     * </p>
     */
    public void testMatches_NullNodeAccuracy() {
        assertFalse("The matches result should be correct.", nodeMatcher.matches(null));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>matches(BreadCrumbNode)</code> with not matched node.
     * </p>
     */
    public void testMatches_NotMatchedNodeAccuracy() {
        assertFalse("The matches result should be correct.", nodeMatcher.matches(new BreadCrumbNode(URL, TITLE)));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>matches(BreadCrumbNode)</code> with matched node.
     * </p>
     */
    public void testMatches_MatchedNodeAccuracy() {
        assertTrue("The matches result should be correct.", nodeMatcher.matches(new BreadCrumbNode("a" + URL, TITLE)));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getNodePattern()</code>.
     * </p>
     */
    public void testGetNodePattern_Accuracy() {
        assertEquals("The nodePattern should be got properly.", NODE_PATTERN, nodeMatcher.getNodePattern());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>equals(Object)</code> with null object.
     * </p>
     */
    public void testEquals_NullObject() {
        assertFalse("The equals result should be correct.", nodeMatcher.equals(null));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>equals(Object)</code> with not RegexNodeMatcher-type object.
     * </p>
     */
    public void testEquals_NotRegexNodeMatcherTypeObject() {
        assertFalse("The equals result should be correct.", nodeMatcher.equals("string"));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>equals(Object)</code> with not equals object.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testEquals_NotEqualsObject() throws Exception {
        assertFalse("The equals result should be correct.",
            nodeMatcher.equals(new RegexNodeMatcher("changed", URL, NODE_PATTERN)));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>equals(Object)</code> with equals object.
     * </p>
     */
    public void testEquals_EqualsObject() {
        assertTrue("The equals result should be correct.", nodeMatcher.equals(nodeMatcher));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>hashCode()</code>.
     * </p>
     */
    public void testHashCode_Accuracy() {
        nodeMatcher.hashCode();
    }
}
